@@ -18,7 +18,7 @@ for LL, (LH, HL, HH) in pywt.swt2(arr, 'bior1.3', level=3, start_level=0):
     fig = plt.figure()
     for i, a in enumerate([LL, LH, HL, HH]):
         ax = fig.add_subplot(2, 2, i + 1)
-        ax.imshow(a, origin='image', interpolation="nearest", cmap=plt.cm.gray)
+        ax.imshow(a, origin='upper', interpolation="nearest", cmap=plt.cm.gray)
         ax.set_title(titles[i], fontsize=12)
 
     fig.suptitle("SWT2 coefficients, level %s" % level, fontsize=14)
