@@ -40,7 +40,8 @@ ax.set_yticks(np.arange(0.5, len(labels) + 0.5), labels)
 # Show spectrogram and wavelet packet coefficients
 fig2 = plt.figure()
 ax2 = fig2.add_subplot(211)
-ax2.specgram(data, NFFT=64, noverlap=32, cmap=cmap)
+ax2.specgram(data, NFFT=64, noverlap=32, Fs=2, cmap=cmap,
+             interpolation='bilinear')
 ax2.set_title("Spectrogram of signal")
 ax3 = fig2.add_subplot(212)
 ax3.imshow(values, origin='upper', extent=[-1, 1, -1, 1],
