@@ -1,4 +1,4 @@
-/* Copyright (c) 2006-2012 Filip Wasilewski <http://en.ig.ma/> 
+/* Copyright (c) 2006-2012 Filip Wasilewski <http://en.ig.ma/>
  * Copyright (c) 2012-2016 The PyWavelets Developers
  *                         <https://github.com/PyWavelets/pywt>
  * See COPYING for license details.
@@ -6,6 +6,11 @@
 
 #include "templating.h"
 
+
+#ifndef REAL_TYPE
+#error REAL_TYPE must be defined here.
+#else
+
 #ifndef TYPE
 #error TYPE must be defined here.
 #else
@@ -37,7 +42,7 @@
  */
 
 int CAT(TYPE, _downsampling_convolution_periodization)(const TYPE * const restrict input, const size_t N,
-                                                       const TYPE * const restrict filter, const size_t F,
+                                                       const REAL_TYPE * const restrict filter, const size_t F,
                                                        TYPE * const restrict output, const size_t step,
                                                        const size_t fstep)
 {
@@ -120,7 +125,7 @@ int CAT(TYPE, _downsampling_convolution_periodization)(const TYPE * const restri
 
 
 int CAT(TYPE, _downsampling_convolution)(const TYPE * const restrict input, const size_t N,
-                                         const TYPE * const restrict filter, const size_t F,
+                                         const REAL_TYPE * const restrict filter, const size_t F,
                                          TYPE * const restrict output,
                                          const size_t step, MODE mode)
 {
@@ -151,7 +156,17 @@ int CAT(TYPE, _downsampling_convolution)(const TYPE * const restrict input, cons
                 for(k = 0; k < N && j < F; ++j, ++k)
                     sum += filter[j]*input[k];
                 for(k = 0; k < N && j < F; ++k, ++j)
-                    sum += filter[j] * input[N-1-k];
+                    sum += filter[j]*input[N-1-k];
+            }
+            break;
+        case MODE_ANTISYMMETRIC:
+            // half-sample anti-symmetric
+            while (j < F){
+                size_t k;
+                for(k = 0; k < N && j < F; ++j, ++k)
+                    sum -= filter[j]*input[k];
+                for(k = 0; k < N && j < F; ++k, ++j)
+                    sum += filter[j]*input[N-1-k];
             }
             break;
         case MODE_REFLECT:
@@ -160,9 +175,28 @@ int CAT(TYPE, _downsampling_convolution)(const TYPE * const restrict input, cons
                 for(k = 1; k < N && j < F; ++j, ++k)
                     sum += filter[j]*input[k];
                 for(k = 1; k < N && j < F; ++k, ++j)
-                    sum += filter[j] * input[N-1-k];
+                    sum += filter[j]*input[N-1-k];
             }
             break;
+        case MODE_ANTIREFLECT:{
+            // whole-sample anti-symmetric
+            size_t k;
+            TYPE le = input[0];    // current left edge value
+            TYPE tmp = 0;
+            while (j < F) {
+                for(k = 1; k < N && j < F; ++j, ++k){
+                    tmp = le - (input[k] - input[0]);
+                    sum += filter[j]*tmp;
+                }
+                le = tmp;
+                for(k = 1; k < N && j < F; ++j, ++k){
+                    tmp = le + (input[N-1-k] - input[N-1]);
+                    sum += filter[j]*tmp;
+                }
+                le = tmp;
+            }
+            break;
+        }
         case MODE_CONSTANT_EDGE:
             for(; j < F; ++j)
                 sum += filter[j]*input[0];
@@ -215,6 +249,16 @@ int CAT(TYPE, _downsampling_convolution)(const TYPE * const restrict input, cons
                     sum += filter[i-N-j]*input[k];
             }
             break;
+        case MODE_ANTISYMMETRIC:
+            // half-sample anti-symmetric
+            while (i - j >= N){
+                size_t k;
+                for(k = 0; k < N && i-j >= N; ++j, ++k)
+                    sum -= filter[i-N-j]*input[N-1-k];
+                for(k = 0; k < N && i-j >= N; ++j, ++k)
+                    sum += filter[i-N-j]*input[k];
+            }
+            break;
         case MODE_REFLECT:
             while (i - j >= N){
                 size_t k;
@@ -224,6 +268,25 @@ int CAT(TYPE, _downsampling_convolution)(const TYPE * const restrict input, cons
                     sum += filter[i-N-j]*input[k];
             }
             break;
+        case MODE_ANTIREFLECT:{
+            // whole-sample anti-symmetric
+            size_t k;
+            TYPE re = input[N-1];    // current right edge value
+            TYPE tmp = 0;
+            while (i - j >= N) {
+                for(k = 1; k < N && i-j >= N; ++j, ++k){
+                    tmp = re - (input[N-1-k] - input[N-1]);
+                    sum += filter[i-N-j]*tmp;
+                }
+                re = tmp;
+                for(k = 1; k < N && i-j >= N; ++j, ++k){
+                    tmp = re + (input[k] - input[0]);
+                    sum += filter[i-N-j]*tmp;
+                }
+                re = tmp;
+            }
+            break;
+        }
         case MODE_CONSTANT_EDGE:
             for(; i-j >= N; ++j)
                 sum += filter[j]*input[N-1];
@@ -257,7 +320,17 @@ int CAT(TYPE, _downsampling_convolution)(const TYPE * const restrict input, cons
                 for(k = 0; k < N && j < F; ++j, ++k)
                     sum += filter[j]*input[k];
                 for(k = 0; k < N && j < F; ++k, ++j)
-                    sum += filter[j] * input[N-1-k];
+                    sum += filter[j]*input[N-1-k];
+            }
+            break;
+        case MODE_ANTISYMMETRIC:
+            // half-sample anti-symmetric
+            while (j < F){
+                size_t k;
+                for(k = 0; k < N && j < F; ++j, ++k)
+                    sum -= filter[j]*input[k];
+                for(k = 0; k < N && j < F; ++k, ++j)
+                    sum += filter[j]*input[N-1-k];
             }
             break;
         case MODE_REFLECT:
@@ -266,9 +339,28 @@ int CAT(TYPE, _downsampling_convolution)(const TYPE * const restrict input, cons
                 for(k = 1; k < N && j < F; ++j, ++k)
                     sum += filter[j]*input[k];
                 for(k = 1; k < N && j < F; ++k, ++j)
-                    sum += filter[j] * input[N-1-k];
+                    sum += filter[j]*input[N-1-k];
             }
             break;
+        case MODE_ANTIREFLECT:{
+            // whole-sample anti-symmetric
+            size_t k;
+            TYPE le = input[0];    // current left edge value
+            TYPE tmp = 0;
+            while (j < F) {
+                for(k = 1; k < N && j < F; ++j, ++k){
+                    tmp = le - (input[k] - input[0]);
+                    sum += filter[j]*tmp;
+                }
+                le = tmp;
+                for(k = 1; k < N && j < F; ++j, ++k){
+                    tmp = le + (input[N-1-k] - input[N-1]);
+                    sum += filter[j]*tmp;
+                }
+                le = tmp;
+            }
+            break;
+            }
         case MODE_CONSTANT_EDGE:
             for(; j < F; ++j)
                 sum += filter[j]*input[0];
@@ -308,6 +400,16 @@ int CAT(TYPE, _downsampling_convolution)(const TYPE * const restrict input, cons
                     sum += filter[i-N-j]*input[k];
             }
             break;
+        case MODE_ANTISYMMETRIC:
+            // half-sample anti-symmetric
+            while (i - j >= N){
+                size_t k;
+                for(k = 0; k < N && i-j >= N; ++j, ++k)
+                    sum -= filter[i-N-j]*input[N-1-k];
+                for(k = 0; k < N && i-j >= N; ++j, ++k)
+                    sum += filter[i-N-j]*input[k];
+            }
+            break;
         case MODE_REFLECT:
             while (i - j >= N){
                 size_t k;
@@ -317,6 +419,27 @@ int CAT(TYPE, _downsampling_convolution)(const TYPE * const restrict input, cons
                     sum += filter[i-N-j]*input[k];
             }
             break;
+        case MODE_ANTIREFLECT:{
+            // whole-sample anti-symmetric
+            size_t k;
+            TYPE re = input[N-1];    //current right edge value
+            TYPE tmp = 0;
+            while (i - j >= N) {
+                //first reflection
+                for(k = 1; k < N && i-j >= N; ++j, ++k){
+                    tmp = re - (input[N-1-k] - input[N-1]);
+                    sum += filter[i-N-j]*tmp;
+                }
+                re = tmp;
+                //second reflection
+                for(k = 1; k < N && i-j >= N; ++j, ++k){
+                    tmp = re + (input[k] - input[0]);
+                    sum += filter[i-N-j]*tmp;
+                }
+                re = tmp;
+            }
+            break;
+            }
         case MODE_CONSTANT_EDGE:
             for(; i-j >= N; ++j)
                 sum += filter[j]*input[N-1];
@@ -347,7 +470,7 @@ int CAT(TYPE, _downsampling_convolution)(const TYPE * const restrict input, cons
 }
 
 int CAT(TYPE, _upsampling_convolution_full)(const TYPE * const restrict input, const size_t N,
-                                            const TYPE * const restrict filter, const size_t F,
+                                            const REAL_TYPE * const restrict filter, const size_t F,
                                             TYPE * const restrict output, const size_t O)
 {
     /* Performs a zero-padded convolution, using each input element for two
@@ -402,7 +525,7 @@ int CAT(TYPE, _upsampling_convolution_full)(const TYPE * const restrict input, c
 
 
 static int CAT(TYPE, _upsampling_convolution_valid_sf_periodization)(const TYPE * const restrict input, const size_t N,
-                                                                     const TYPE * const restrict filter, const size_t F,
+                                                                     const REAL_TYPE * const restrict filter, const size_t F,
                                                                      TYPE * const restrict output, const size_t O)
 {
     // TODO? Allow for non-2 step
@@ -514,7 +637,7 @@ static int CAT(TYPE, _upsampling_convolution_valid_sf_periodization)(const TYPE
  */
 
 int CAT(TYPE, _upsampling_convolution_valid_sf)(const TYPE * const restrict input, const size_t N,
-                                                const TYPE * const restrict filter, const size_t F,
+                                                const REAL_TYPE * const restrict filter, const size_t F,
                                                 TYPE * const restrict output, const size_t O,
                                                 MODE mode)
 {
@@ -545,13 +668,14 @@ int CAT(TYPE, _upsampling_convolution_valid_sf)(const TYPE * const restrict inpu
 }
 
 /* -> swt - todo */
-int CAT(TYPE, _upsampled_filter_convolution)(const TYPE* input, const size_t N,
-                                             const TYPE* filter, const size_t F,
-                                             TYPE* output,
+int CAT(TYPE, _upsampled_filter_convolution)(const TYPE * const restrict input, const size_t N,
+                                             const REAL_TYPE * const restrict filter, const size_t F,
+                                             TYPE * const restrict output,
                                              const size_t step, MODE mode)
 {
     return -1;
 }
 
 #undef restrict
+#endif /* REAL_TYPE */
 #endif /* TYPE */
